#include <Arduino.h>
#include <U8g2lib.h>
#include <arduinoFFT.h>

#ifdef U8X8_HAVE_HW_SPI
#include <SPI.h>
#endif

U8G2_SH1107_64X128_F_4W_HW_SPI u8g2(U8G2_R1, /* cs=*/ 14, /* dc=*/ 27, /* reset=*/ 33);

arduinoFFT FFT = arduinoFFT();
hw_timer_t *timer = NULL;
bool getSample;
const byte maxSamples = 128;
const word samplingFrequency = 2000;
double vReal[maxSamples];
double vImag[maxSamples];

word adcData[maxSamples];
byte sampleCount;
bool adcDataReady;

void timerInterrupt() {
  if (!getSample) {
    getSample = true;
  }
}

void setup() {
  u8g2.begin();
  u8g2.setFont(u8g2_font_5x7_tr);
  analogReadResolution(12);
  getSample = false;
  adcDataReady = false;
  sampleCount = 0;
  timer = timerBegin(0, 80, true);
  timerAttachInterrupt(timer, &timerInterrupt, true);
  timerAlarmWrite(timer, 1000000 / samplingFrequency, true);
  timerAlarmEnable(timer);
}

void readData(void)
{
  if (getSample && !adcDataReady) {
    adcData[sampleCount] = analogRead(35);
    sampleCount++;
    if (sampleCount >= maxSamples) {
      adcDataReady = true;
      sampleCount = 0;
    } else {
      getSample = false;
    }
  }
}

word getFrequency(void)
{
  for (byte i = 0; i < maxSamples; i++) {
    vReal[i] = double(adcData[i]);
    vImag[i] = 0;
  }
  FFT.Windowing(vReal, maxSamples, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
  FFT.Compute(vReal, vImag, maxSamples, FFT_FORWARD);
  FFT.ComplexToMagnitude(vReal, vImag, maxSamples);
  return FFT.MajorPeak(vReal, maxSamples, samplingFrequency);
}

word getFlicker(void)
{
  word flicker = 0;
  word minVal = 65535, maxVal = 0;
  for (byte i = 0; i < maxSamples; i++)
  {
    if (minVal > adcData[i]) {
      minVal = adcData[i];
    }
    if (maxVal < adcData[i]) {
      maxVal = adcData[i];
    }
  }
  if ((minVal + maxVal) != 0) {
    flicker = ((maxVal - minVal) / (maxVal + minVal)) * 100;
  }
  return flicker;
}

void drawPlot(void)
{
  word r = 1 << 12;
  r = r / 54;
  for (byte i = 0; i < maxSamples; i++) {
    u8g2.drawPixel(i, 54 - (adcData[i] / r));
  }
}

void loop() {
  readData();
Frequency: 
  if (adcDataReady) {
    char displayText[20];
    word frequency = getFrequency();
    word flicker = getFlicker();
    u8g2.clearBuffer();
    snprintf (displayText, 19, "F:%dHz", frequency);
    u8g2.drawStr(10, 8, displayText);
    snprintf (displayText, 19, "Flicker:%d%%", flicker);
    u8g2.drawStr(60, 8, displayText);
    u8g2.drawLine(0, 10, 163, 10);
    drawPlot();
    u8g2.sendBuffer();
    adcDataReady = false;
    getSample = false;
    delay(100);
  }
}
